# Disable LTO until upstream fixes GDB's ODR woes.
%define _lto_cflags %{nil}

%global bundledsrc 20220501
# timestamp of source gnulib installed into gnulib/.
# gnulib/ has the commit id, get it's date in gnulib gitrepo.
%global bundlegnulib 20210105

%global gdb_srcfile gdb-%{version}
%global gdb_build_target build-%{_target_platform}

# Installed (but unpackaged) file(s) found: /usr/lib/debug/usr/bin/gdb-gdb.py, have to disable subpkg-debuginfo
%undefine _debuginfo_subpackages

%global librpmver 9


Summary: A GNU source-level debugger for C, C++, Fortran, Go and other languages
Name: gdb
Version: 13.2
Release: 5%{?dist}
License: GPLv3+ and GPLv3+ with exceptions and GPLv2+ and GPLv2+ with exceptions and GPL+ and LGPLv2+ and LGPLv3+ and BSD and Public Domain and GFDL
URL: https://gnu.org/software/gdb/
Source0: https://ftp.gnu.org/gnu/gdb/gdb-%{version}.tar.xz
# Man page for gstack(1).
Source1: gdb-gstack.man
# /etc/gdbinit
Source2: gdbinit

# combine long list patch into one file
Patch3998: _gdb.spec.Patch.include
Patch3999: _gdb.spec.patch.include
%include %{PATCH3998}


BuildRequires: rpm-libs gcc-c++
BuildRequires: readline-devel >= 7.0
BuildRequires: ncurses-devel texinfo gettext flex bison
BuildRequires: expat-devel xz-devel
BuildRequires: rpm-devel
BuildRequires: zlib-devel libselinux-devel python3-devel
BuildRequires: /usr/bin/pod2man
BuildRequires: libbabeltrace-devel
%global have_libipt 0
%ifarch x86_64
%global have_libipt 1
BuildRequires: libipt-devel
%endif
BuildRequires: mpfr-devel
BuildRequires: source-highlight-devel
BuildRequires: xxhash-devel make boost-devel

Requires:   gdb-headless = %{version}-%{release}
Recommends: gcc-gdb-plugin
Recommends: dnf-command(debuginfo-install)
# https://bugzilla.redhat.com/show_bug.cgi?id=1209492
Recommends: default-yama-scope

# https://pagure.io/packaging-committee/issue/43 #issue/109
Provides: bundled(libiberty) = %{bundledsrc}
Provides: bundled(gnulib) = %{bundlegnulib}
# https://pagure.io/packaging-committee/issue/109
Provides: bundled(binutils) = %{bundledsrc}
# https://pagure.io/packaging-committee/issue/130
Provides: bundled(md5-gcc) = %{bundledsrc}

# remove gdb-headless from 12.1-1, add conflict
Conflicts: gdb  < 12.1-1

%description
GDB, the GNU Project debugger, allows you to see what is going 
on 'inside' another program while it executes -- or what another
program was doing at the moment it crashed.


%package headless
Summary: Headless package for gdb

%description headless
Headless package for gdb.


%package minimal
Summary: A GNU source-level debugger for C, C++, Fortran, Go and other languages (minimal version)
# gdb-add-index is shared with gdb-headless and it must be from same version
Conflicts: %{name}-headless < %{version}-%{release}
Conflicts: %{name}-headless > %{version}-%{release}

%description minimal
This package provides a minimal version of GDB, only being used by buildroot.
It should probably not be used by end users.


%package gdbserver
Summary: A standalone server for GDB (the GNU debugger)

%description gdbserver
GDB, the GNU Project debugger, allows you to see what is going 
on 'inside' another program while it executes -- or what another
program was doing at the moment it crashed.

This package provides a program that allows you to run GDB on a different
machine than the one which is running the program being debugged.


%package doc
Summary: Documentation for GDB (the GNU debugger)
License: GFDL
BuildArch: noarch

%description doc
GDB, the GNU Project debugger, allows you to see what is going 
on 'inside' another program while it executes -- or what another
program was doing at the moment it crashed.

This package provides INFO, HTML user manual for GDB.

%prep
%setup -q -n %{gdb_srcfile}

(cd gdb;rm -fv $(perl -pe 's/\\\n/ /' <Makefile.in|sed -n 's/^YYFILES = //p'))

# *.info* would be built in compile, delete old ones here.
find -name "*.info*"|xargs rm -f

# Apply patches
%include %{PATCH3999}


# specify the version printed at GDB startup
cat > gdb/version.in << _FOO
%{_vendor} %{version}-%{release}
_FOO

# Remove files generated in FSF release process.
rm -f libdecnumber/gstdint.h
rm -f bfd/doc/*.info
rm -f bfd/doc/*.info-*
rm -f gdb/doc/*.info
rm -f gdb/doc/*.info-*

mv -f readline/readline/doc readline-doc
rm -rf readline/readline/*
mv -f readline-doc readline/readline/doc

# use zlib-devel instead of zlib in gdb, the same like texinfo
rm -rf zlib texinfo

%build
rm -rf %{buildroot}

test -e %{_libdir}/librpm.so.%{librpmver}

COMMON_GDB_CONFIGURE_FLAGS="\
	--prefix=%{_prefix}					\
	--libdir=%{_libdir}					\
	--sysconfdir=%{_sysconfdir}				\
	--mandir=%{_mandir}					\
	--infodir=%{_infodir}					\
	--with-gdb-datadir=%{_datadir}/gdb			\
	--enable-gdb-build-warnings=,-Wno-unused,-Wno-deprecated-declarations,-Wno-unused-function \
	--enable-build-with-cxx					\
%ifnarch x86_64 ppc64le aarch64
	--disable-werror					\
%else
	--enable-werror						\
%endif
	--with-separate-debug-dir=/usr/lib/debug		\
	--disable-sim						\
	--disable-rpath						\
	--without-stage1-ldflags				\
	--disable-libmcheck					\
	--with-system-readline					\
	--without-libunwind					\
	--enable-64-bit-bfd					\
	--with-mpfr						\
	--with-system-zlib					\
	--with-lzma						\
        --without-debuginfod					\
        --without-guile                                         \
	--disable-libctf
"

# build gdb-mininal
mkdir %{gdb_build_target}-minimal
cd %{gdb_build_target}-minimal

export CFLAGS="$RPM_OPT_FLAGS %{?_with_asan:-fsanitize=address}"
export LDFLAGS="%{?__global_ldflags} %{?_with_asan:-fsanitize=address}"
export CXXFLAGS="$CFLAGS"

../configure ${COMMON_GDB_CONFIGURE_FLAGS} \
        --without-babeltrace               \
        --without-expat                    \
        --disable-tui                      \
        --without-python                   \
        --disable-inprocess-agent          \
        --without-intel-pt                 \
        --disable-unit-tests               \
        --disable-source-highlight         \
        --with-auto-load-dir='$debugdir:$datadir/auto-load'	  \
	--with-auto-load-safe-path='$debugdir:$datadir/auto-load' \
	--enable-targets=aarch64-linux-gnu \
	%{_target_platform}

# Prepare gdb/config.h
%make_build CFLAGS="$CFLAGS" LDFLAGS="$LDFLAGS" V=1 maybe-configure-gdb
perl -i.relocatable -pe 's/^(D\[".*_RELOCATABLE"\]=" )1(")$/${1}0$2/' gdb/config.status
%make_build CFLAGS="$CFLAGS" LDFLAGS="$LDFLAGS" V=1

cd ..

# build gdb all
mkdir %{gdb_build_target}
cd %{gdb_build_target}

export CFLAGS="$RPM_OPT_FLAGS %{?_with_asan:-fsanitize=address}"
export LDFLAGS="%{?__global_ldflags} %{?_with_asan:-fsanitize=address}"
CFLAGS="$CFLAGS -DDNF_DEBUGINFO_INSTALL"
export CXXFLAGS="$CFLAGS"

../configure ${COMMON_GDB_CONFIGURE_FLAGS}                        \
        --with-system-gdbinit=%{_sysconfdir}/gdbinit              \
        --with-babeltrace                                         \
        --with-expat                                              \
        --without-libexpat-prefix                                 \
        --enable-tui                                              \
        --with-python=%{__python3}                                \
%ifnarch loongarch64
        --enable-inprocess-agent                                  \
%endif
%if %{have_libipt}
        --with-intel-pt                                           \
%else
        --without-intel-pt                                        \
%endif
        --with-xxhash                                             \
        --with-rpm=librpm.so.%{librpmver}                         \
        --enable-unit-tests                                       \
	--with-auto-load-dir='$debugdir:$datadir/auto-load'	  \
	--with-auto-load-safe-path='$debugdir:$datadir/auto-load' \
	--enable-targets=powerpc-linux-gnu,aarch64-linux-gnu	  \
	%{_target_platform}

# modify gdb/config.h, diable relocatable
%make_build CFLAGS="$CFLAGS" LDFLAGS="$LDFLAGS" V=1 maybe-configure-gdb
perl -i.relocatable -pe 's/^(D\[".*_RELOCATABLE"\]=" )1(")$/${1}0$2/' gdb/config.status

%make_build CFLAGS="$CFLAGS" LDFLAGS="$LDFLAGS" V=1

# make sure set all relocatable to 0.
! grep '_RELOCATABLE.*1' gdb/config.h

cd ..
cd %{gdb_build_target}

%make_build \
     -C gdb/doc {gdb,annotate}{.info,/index.html} MAKEHTMLFLAGS=--no-split MAKEINFOFLAGS=--no-split V=1

cp $RPM_BUILD_DIR/%{gdb_srcfile}/gdb/NEWS $RPM_BUILD_DIR/%{gdb_srcfile}


%check
cd %{gdb_build_target}
(cd gdb; make run GDBFLAGS='-batch -ex "maintenance selftest"')
echo ====================TESTSUITE DISABLED=========================


%install
# install gdb-minimal
cd %{gdb_build_target}-minimal
rm -rf $RPM_BUILD_ROOT
%make_install %{?_smp_mflags}
# Delete everything except the 'gdb' binary, and then rename it to 'gdb.minimal'.
rm -rfv $RPM_BUILD_ROOT%{_prefix}/{include,lib*,share}
rm -fv $RPM_BUILD_ROOT%{_bindir}/{gcore,gdbserver,gstack,gdb-add-index}
mv $RPM_BUILD_ROOT%{_bindir}/gdb $RPM_BUILD_ROOT%{_bindir}/gdb.minimal
cd ..

#install gdb of all build.
cd %{gdb_build_target}
%make_install %{?_smp_mflags}

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit.d
touch -r %{SOURCE2} $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit.d
sed 's#%%{_sysconfdir}#%{_sysconfdir}#g' <%{SOURCE2} >$RPM_BUILD_ROOT%{_sysconfdir}/gdbinit
touch -r %{SOURCE2} $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit

for i in `find $RPM_BUILD_ROOT%{_datadir}/gdb/python/gdb -name "*.py"`
do
  # set timestamp to current time
  touch -r $RPM_BUILD_DIR/%{gdb_srcfile}/gdb/version.in $i
done

%if 0%{?_enable_debug_packages:1}
mkdir -p $RPM_BUILD_ROOT/usr/lib/debug%{_bindir}
cp -p ./gdb/gdb-gdb.py $RPM_BUILD_ROOT/usr/lib/debug%{_bindir}/
for pyo in "" "-O";do
  %{__python3} $pyo -c 'import compileall, re, sys; sys.exit (not compileall.compile_dir("'"$RPM_BUILD_ROOT/usr/lib/debug%{_bindir}"'", 1, "'"/usr/lib/debug%{_bindir}"'"))'
done
%endif

%py_byte_compile %{__python3} %{buildroot}%{_datadir}/gdb/python/gdb


for i in $(echo sbin bin lib $(basename %{_libdir}) | tr ' ' '\n' | sort -u);do
  mkdir -p $RPM_BUILD_ROOT%{_datadir}/gdb/auto-load/%{_prefix}/$i
  ln -s $(echo %{_prefix}|sed 's#^/*##')/$i \
        $RPM_BUILD_ROOT%{_datadir}/gdb/auto-load/$i
done
for i in `find $RPM_BUILD_ROOT%{_datadir}/gdb -name "*.py"`; do
  # set timestamp to current time
  touch -r $RPM_BUILD_DIR/%{gdb_srcfile}/gdb/version.in $i
done


# rm the files owned by other packages.
rm -rf $RPM_BUILD_ROOT%{_datadir}/locale/
rm -f $RPM_BUILD_ROOT%{_infodir}/bfd*
rm -f $RPM_BUILD_ROOT%{_infodir}/standard*
rm -f $RPM_BUILD_ROOT%{_infodir}/configure*
rm -f $RPM_BUILD_ROOT%{_infodir}/sframe-spec*
# just provide jit-reader.h.
rm -rf $RPM_BUILD_ROOT%{_includedir}/*.h
rm -rf $RPM_BUILD_ROOT/%{_libdir}/lib{bfd*,opcodes*,iberty*,ctf*,sframe*}

# pstack related
cp -p %{SOURCE1} $RPM_BUILD_ROOT%{_mandir}/man1/gstack.1
ln -s gstack.1 $RPM_BUILD_ROOT%{_mandir}/man1/pstack.1
ln -s gstack $RPM_BUILD_ROOT%{_bindir}/pstack

# rm cross-target files
(cd $RPM_BUILD_ROOT%{_datadir}/gdb/syscalls
 rm -f mips*.xml
 rm -f sparc*.xml
 rm -f ppc{,64}-linux.xml

%ifnarch x86_64
 rm -f {amd64,i386}-linux.xml
%endif

%ifnarch ppc64le
 rm -f ppc64le-linux.xml
%endif
)

# delete development Documentation.
rm -f $RPM_BUILD_ROOT%{_infodir}/gdbint*
rm -f $RPM_BUILD_ROOT%{_infodir}/stabs*
rm -f $RPM_BUILD_ROOT%{_infodir}/ctf-spec*

# Delete the dir file that would be updated at rpm installation.
rm -f $RPM_BUILD_ROOT%{_infodir}/dir

# unrelated files.
rm -f $RPM_BUILD_ROOT%{_datadir}/gdb/system-gdbinit/elinos.py
rm -f $RPM_BUILD_ROOT%{_datadir}/gdb/system-gdbinit/wrs-linux.py
rmdir $RPM_BUILD_ROOT%{_datadir}/gdb/system-gdbinit


%files
%license COPYING3 COPYING COPYING.LIB COPYING3.LIB
%doc README NEWS
%{_bindir}/gdb
%{_bindir}/gcore
%{_bindir}/gstack
%{_bindir}/pstack
# Provide gdb/jit-reader.h to enable users to write customized GDB JIT plugins.
%{_includedir}/gdb
%{_datadir}/gdb
%{_mandir}/*/gdb.1*
%{_mandir}/*/gcore.1*
%{_mandir}/*/gstack.1*
%{_mandir}/*/pstack.1*


%files headless
%config(noreplace) %{_sysconfdir}/gdbinit
%{_sysconfdir}/gdbinit.d
%{_bindir}/gdb-add-index
%{_mandir}/*/gdbinit.5*
%{_mandir}/*/gdb-add-index.1*

%files minimal
%{_bindir}/gdb.minimal
%{_bindir}/gdb-add-index


%files gdbserver
%{_bindir}/gdbserver
%ifnarch loongarch64
%{_libdir}/libinproctrace.so
%endif
%{_mandir}/*/gdbserver.1*

%pre
for i in $(echo bin lib $(basename %{_libdir}) sbin|tr ' ' '\n'|sort -u);do
  src="%{_datadir}/gdb/auto-load/$i"
  dst="%{_datadir}/gdb/auto-load/%{_prefix}/$i"
  if test -d $src -a ! -L $src;then
    if ! rmdir 2>/dev/null $src;then
      mv -n $src/* $dst/
      rmdir $src
    fi
  fi
done


%files doc
%doc %{gdb_build_target}/gdb/doc/{gdb,annotate}.html
%{_infodir}/annotate.info*
%{_infodir}/gdb.info*


%changelog
* Tue Mar 19 2024 Wenlong Zhang <zhangwenlong@loongson.cn> - 13.2-5
- fix build error for loongarch64

* Thu Sep 21 2023 rockerzhu <rockerzhu@tencent.com> - 13.2-4
- Disable guile.

* Tue Sep 19 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 13.2-3
- Rebuilt for python 3.11

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 13.2-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 22 2023 Shuo Wang <abushwang@tencent.com> - 13.2-1
- update to 13.2

* Wed Aug 02 2023 rockerzhu <rockerzhu@tencent.com> - 12.1-7
- Fix /usr/bin/gdb-add-index to use gdb.minimal.

* Wed Aug 02 2023 rockerzhu <rockerzhu@tencent.com> - 12.1-6
- Add sub-package of minimal to provides /usr/bin/gdb-add-index(needed by rpm-build) and
  gdb.minimal(needed by /usr/bin/gdb-add-index)
- Rebuilt with boost 1.82.0.

* Mon Jul 31 2023 rockerzhu <rockerzhu@tencent.com> - 12.1-5
- Add sub-package of headless, avoiding rpm-build to require all gdb, but to require gdb-headless
  and not to require source-highlight.

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 12.1-4
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 12.1-3
- Rebuilt for OpenCloudOS Stream 23

* Thu Feb 23 2023 Yi Lin <nilusyi@tencent.com> - 12.1-2
- use condition macro to define vendor.

* Mon Jun 6 2022 Yi Lin <nilusyi@tencent.com> - 12.1-1
- package init.
- remove condition compile: buildisa, scl, testsuite
- remove subpkg: gdb-headless, gdb-minimal
- remove pdf generate. remove its requirement
- remove condition compile --with-fprofile --without-python --with asan
